/*
	GLDemo is a test shell program for the Gublists class library.
	It demonstrates the use of the Timeline and Flexpipe classes,
	in a not particularly realistic way.

	Author: Scott Burleigh
	Copyright 2003 by Steps On Cats.
									*/
import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import com.stepsoncats.gublists.*;

class GLDemoListener extends Thread
{
	private DatagramSocket	sock;
	private ByteArrayPipe	pipeToShell;

	public void	run()
	{
		byte		stringBuffer[] = new byte[65536];
		DatagramPacket	pkt = new DatagramPacket(stringBuffer,
					stringBuffer.length);

		//	Start message handling loop.

		while (true)
		{
			try
			{
				//	Get message from peer.

				sock.receive(pkt);

				//	Deliver message content.

				int	len = pkt.getLength();
				byte[]	msg = new byte[len];
				System.arraycopy(pkt.getData(), 0, msg, 0, len);
				pipeToShell.write(msg);

				//	Prepare to receive next message.

				pkt.setLength(stringBuffer.length);
			}
			catch (Exception x1)
			{
				System.out.println ("Exception in 'listener' "
					+ "thread of GLDemo: "
					+ x1.getMessage());
				x1.printStackTrace();
				System.exit(-1);
			}
		}
	}

	//	Instance constructor.

	GLDemoListener(DatagramSocket sock, ByteArrayPipe pipeToShell)
	{
		super("gldemo_listener");
		this.sock = sock;
		this.pipeToShell = pipeToShell;
	}
}

class MsgEvent extends TimelineEvent
{
	private DatagramSocket	sock;
	private String		line;
	private InetAddress	peerHost;
	private int		peerPortNbr;

	public long	handleEvent()
	{
		try
		{
			byte[]	msg = line.getBytes();
			sock.send(new DatagramPacket(msg, msg.length,
					peerHost, peerPortNbr));
		}
		catch (IOException x1)
		{
			System.out.println("Can't send datagram: "
					+ x1.getMessage());
			System.exit(-1);
		}

		return 0;
	}

	//	Instance constructor.

	MsgEvent(long interval, DatagramSocket sock, String line,
			InetAddress peerHost, int peerPortNbr)
	{
		super(interval);
		this.sock = sock;
		this.line = line;
		this.peerHost = peerHost;
		this.peerPortNbr = peerPortNbr;
	}
}

public class GLDemo
{
	public static void main(String[] args)
	{
		int		ownPortNbr;
		int		peerPortNbr = 0;
		InetAddress	localHost;
		InetAddress	peerHost = null;
		DatagramSocket	sock = null;
		ByteArrayPipe	pipe;
		Timeline	timeline;
		GLDemoListener	listener;
		BufferedReader	console;

		if (args.length < 2)
		{
			System.out.println("Usage:  GLDemo <own port nbr> "
					+ "<peer port nbr> "
					+ "[<peer host name>]");
			System.exit(0);
		}

		try
		{
			ownPortNbr = Integer.parseInt(args[0]);
			peerPortNbr = Integer.parseInt(args[1]);
			localHost = InetAddress.getLocalHost();
			if (args.length == 2)
			{
				peerHost = localHost;
			}
			else
			{
				peerHost = InetAddress.getByName(args[2]);
			}

			sock = new DatagramSocket(ownPortNbr);
		}
		catch (Exception x1)
		{
			System.out.println("Can't open outbound socket: "
					+ x1.getMessage());
			System.exit(-1);
		}

		pipe = new ByteArrayPipe();
		pipe.attach();	//	Register as pipe's reader.
		timeline = new Timeline();
		timeline.activate();
		listener = new GLDemoListener(sock, pipe);
		listener.setDaemon(true);
		listener.start();
		console = new BufferedReader(new InputStreamReader(System.in));
		while (true)
		{
			try
			{
				System.out.print(": ");
				String	line = console.readLine();
				if (line.length() == 0)
				{
					byte[]	msg = pipe.poll();
					if (msg == null)
					{
						continue;
					}

					System.out.println("got '"
						+ new String(msg) + "'");
				}
				else
				{
					int	interval = line.length() * 1000;
					timeline.schedule(new MsgEvent(interval,
						sock, line, peerHost,
						peerPortNbr));
				}
			}
			catch (IOException x1)
			{
				System.out.println("Can't read from console: "
					+ x1.getMessage());
				System.exit(-1);
			}
			catch (InterruptedException x2)
			{
				System.out.println("Pipe thread interrupted.");
				System.exit(-1);
			}
		}
	}
}
